import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import javax.swing.*;

public class Tile extends JLabel implements MouseListener {
  private static final long serialVersionUID = -6873580061751396083L;

  private Board board;
  private BufferedImage image;

  private boolean uncovered;
  private boolean flagged;
  private boolean unsure;
  private boolean dud;

  private int row;
  private int col;
  private int value; // -1~8 (-1: mine)

  private final int SIZE = 16;

  public Tile(int row, int col, Board board) {
    super("", JLabel.CENTER);
    this.board = board;
    this.image = this.board.getMain().getImage().getTile();
    this.uncovered = false;
    this.flagged = false;
    this.unsure = false;
    this.dud = false;
    this.row = row;
    this.col = col;
    this.setValue(0);
    this.setTileIcon("covered");
    addMouseListener(this);
    setVerticalAlignment(JLabel.CENTER);
    setHorizontalAlignment(JLabel.CENTER);
    setPreferredSize(new Dimension(this.SIZE, this.SIZE));
  }

  public int topMargin(String state) {
    String[] states = {
      "covered",
      "flagged",
      "unsure",
      "-1",
      "misflagged",
      "covered mine",
      "unsure down",
      "8",
      "7",
      "6",
      "5",
      "4",
      "3",
      "2",
      "1",
      "0"
    };
    for (int i = 0; i < states.length; i += 1) {
      if (states[i].equals(state)) {
        return i;
      }
    }
    return 6;
  }

  public void setTileIcon(String state) {
    setIcon(
        new ImageIcon(
            this.image.getSubimage(0, this.SIZE * this.topMargin(state), this.SIZE, this.SIZE)));
  }

  public int getValue() {
    return this.value;
  }

  public boolean isMine() {
    return (this.value == -1 || this.dud);
  }

  public boolean isUncovered() {
    return this.uncovered;
  }

  public boolean isFlagged() {
    return this.flagged;
  }

  public boolean isUnsure() {
    return this.unsure;
  }

  public boolean isDud() {
    return this.dud;
  }

  public void setValue(int value) {
    this.value = value;
  }

  public void setMine() {
    this.value = -1;
  }

  public void setDud() {
    this.dud = true;
  }

  public void uncover(boolean expanding) {
    this.uncovered = true;
    Minesweeper main = this.board.getMain();
    if (this.value >= 0 || this.isDud()) {
      if (this.isDud()) {
        this.setTileIcon("unsure down");
      } else {
        this.setTileIcon(String.valueOf(this.value));
      }
      if (this.value == 0 && !this.isDud()) {
        this.board.expand(row, col);
      }
      if (main.getStorage().isCreepy()
          && !this.isDud()
          && !expanding
          && Math.round((float) Math.random() * 10) == 0) {
        main.getPlay().boom();
      }
      this.board.checkWon(expanding);
    } else {
      this.setTileIcon("-1");
      main.delGame();
    }
  }

  public void uncover() {
    this.uncover(false);
  }

  public void toggleFlagged() {
    Info info = this.board.getMain().getInfo();
    if (!this.flagged && !this.unsure) {
      this.flagged = true;
      this.setTileIcon("flagged");
      info.getCounter().decrease();
    } else if (this.flagged) {
      this.flagged = false;
      this.unsure = true;
      this.setTileIcon("unsure");
      info.getCounter().increase();
    } else {
      this.flagged = false;
      this.setTileIcon("covered");
      if (!this.unsure) {
        info.getCounter().increase();
      }
      this.unsure = false;
    }
  }

  @Override
  public void mousePressed(MouseEvent e) {
    Minesweeper main = this.board.getMain();
    if (!main.isGameStarted()) {
      main.startGame();
      main.getInfo().getTimer().start();
      if (main.getStorage().isSoundEnabled() && !main.getStorage().isCreepy()) {
        main.getPlay().tick();
      }
    }
    if (SwingUtilities.isLeftMouseButton(e) && !this.uncovered && !this.flagged) {
      this.setTileIcon("0");
    } else if (SwingUtilities.isMiddleMouseButton(e)) {
      if (!this.flagged) {
        this.setTileIcon("0");
      }
      main.getInfo().getSmiley().setSmileyIcon("oops");
    }
  }

  @Override
  public void mouseReleased(MouseEvent e) {
    this.board.getMain().getInfo().getSmiley().setSmileyIcon("smile");
    if ((SwingUtilities.isLeftMouseButton(e) || SwingUtilities.isMiddleMouseButton(e))
        && !this.flagged) {
      this.uncover();
    } else if (SwingUtilities.isRightMouseButton(e) && !this.uncovered) {
      this.toggleFlagged();
    }
  }

  @Override
  public void mouseExited(MouseEvent e) {}

  @Override
  public void mouseEntered(MouseEvent e) {}

  @Override
  public void mouseClicked(MouseEvent e) {}
}
